require 'rbconfig'
require 'fileutils'
require 'lucent'
require 'lucent/assert'
require 'lucent/cli'

module Lucent
  module API

    def run_standard(command)
      run(command) do |process|
        stop_process(process)
      end
    end

    def run(command)
      @commands ||= []
      @commands << command
      puts "DEBUG: @commands = #{@commands}"

      command = create_ruby_command(command)
      puts "DEBUG: (ruby) command = #{command}"

      puts "DEBUG: Lucent.process = #{Lucent.process}"

      in_lucent_directory do
        process = Lucent.process.new(command)

        puts "DEBUG: Process = #{process.inspect}"

        register_process(command, process)
        process.run

        puts "DEBUG: Block given for run standard? #{block_given?}"

        block_given? ? yield(process) : process
      end
    end

    def last_exit_status
      return @last_exit_status if @last_exit_status
      stop_processes
      @last_exit_status
    end

    def all_output
      all_stdout << all_stderr
    end

    def all_stdout
      stop_processes
      only_processes.inject('') { |out, ps| out << ps.stdout }
    end

    def all_stderr
      stop_processes
      only_processes.inject('') { |out, ps| out << ps.stderr }
    end

    def processes
      @processes ||= []
    end

    def register_process(command, process)
      processes << [command, process]
    end

    def stop_process(process)
      @last_exit_status = process.stop
    end

    def stop_processes
      processes.each do |command, process|
        stop_process(process)
      end
    end

    def only_processes
      processes.collect { |command, process| process }
    end

    # @was in_current_dir
    def in_lucent_directory(&block)
      create_path_for(lucent_directory)
      Dir.chdir(lucent_directory, &block)
    end

    # @was current_dir
    def lucent_directory
      File.join(*lucent_path)
    end

    # @was dirs
    # Note: @path was @dirs
    def lucent_path
      @path ||= %w(tmp lucent)
    end

    # @was _mkdir
    def create_path_for(name)
      FileUtils.mkdir_p(name) unless File.directory?(name)
    end

    def unescape(string)
      string = string.gsub(/\e\[\d+(?>(;\d+)*)m/, '')
      string
    end

    def create_ruby_command(command)
      if command =~ /^ruby\s/
        command.gsub(/^ruby\s/, "#{system_ruby} ")
      else
        command
      end
    end

    def system_ruby
      File.join(RbConfig::CONFIG['bindir'], RbConfig::CONFIG['ruby_install_name'])
    end

  end
end