require 'rspec/expectations'

module Lucent
  module API
    include RSpec::Matchers

    def assert_exit_status_and_output(expect_pass, expected, expect_exact)
      assert_success(expect_pass)
      if expect_exact
        assert_exact_output(expected, all_output)
      else
        assert_partial_output(expected, all_output)
      end
    end

    def assert_failing_with(expected)
      assert_success(false)
      assert_output(expected, all_output)
    end

    def assert_success(expect_success)
      expect_success ? assert_exit_status(0) : assert_not_exit_status(0)
    end

    def assert_output(expected, actual)
      actual.should include(expected)
    end

    def assert_partial_output(expected, actual)
      unescape(actual).should include(unescape(expected))
    end

    def assert_exact_output(expected, actual)
      unescape(actual).should == unescape(expected)
    end

    def assert_exit_status(value)
      last_exit_status.should eq(value), "Exit status was #{last_exit_status} but expected it to be #{value}."
    end

    def assert_not_exit_status(value)
      last_exit_status.should_not eq(value), "Exit status was #{last_exit_status} which was not expected."
    end

  end
end